/**
 @author Dmitriy I. Suyasov
 @since 19.08.2005 13:39:11
 */
package dsuyasov.inetDoc.rulesZone;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

public abstract class Rule {

    public static final int RULES = 0;
    public static final int STRING = 1;
    public static final int CHOICE = 2;
    public static final int SPECIAL = 3;

    protected Object object = null;
    protected Integer start = null,
                      end = null;
    protected Collection results = new ArrayList();
    protected ForwardElement nextStep = null;

    private int counter = 0;
    private int ruleNumber = 0;

    public Rule(Object object, int next, int number) throws RuleException {
        if (object == null)
            throw new RuleException("  :  ["+object+"]    .");
        this.object = object;
        start = new Integer(1);
        end = new Integer(1);
        nextStep = new ForwardElement(next, "", null, new ArrayList());
        ruleNumber = number;
    }

    public void setRange(Integer numStart, Integer numEnd) throws RuleException {
        if (numStart == null)
            throw new RuleException("  :   ["+numStart+"]    .");
        start = numStart;
        end = numEnd;
    }

    public void setNext(int next) {
        if (nextStep == null)
            nextStep = new ForwardElement(next, null, null, new ArrayList());
        else
            nextStep.setNext(next);
    }

    public void setText(String text) throws RuleException {
        if (nextStep == null)
            throw new RuleException("    :        .");
        nextStep.setText(text);
    }

    public abstract boolean proceed(ForwardElement element);

    public boolean proceed(String text) {
        return proceed(new ForwardElement(0, text, null, new ArrayList()));
    }

    public static char createSpecialChar(char str) {
        switch (str) {
            case 'n': return '\n';
            case 't': return '\t';
            case 'r': return '\r';
            case 'b': return '\b';
            case 's': return ' ';
            default: return str;
        }
    }

    public void fillZeroTheCounter() {
        counter = 0;
    }

    public void increaseTheCounter() {
        counter++;
    }

    public boolean counterLess() {
        return counter < start.intValue();
    }

    public boolean counterMore() {
        return (end != null) && counter >= end.intValue();
    }

    public int getRuleNumber() {
        return ruleNumber;
    }

    public String getResult() {
        if ((results == null) || (results.size() == 0))
            return null;
        ForwardElement element = (ForwardElement) results.iterator().next();
        return (element.getResult() == null)?element.getText():element.getResult();
    }

    public String getHistory() {
        if ((results == null) || (results.size() == 0))
            return null;
        ForwardElement element = (ForwardElement) results.iterator().next();
        String result = "";
        for (Iterator iter = element.getHistory().iterator(); iter.hasNext(); ) {
            Object[] obj = (Object[]) iter.next();
            String str = (String) obj[2];
            str = str.replace('\n', ' ');
            str = str.replace('\r', ' ');
            str = str.replace('\t', ' ');
            str = (str.length() > 20)?("..."+str.substring(str.length() - 20)):str;
            switch (((Integer) obj[0]).intValue()) {
                case STRING: result = "\t  STRING ["+obj[1]+"]: "+str+"\n"+ result; break;
                case CHOICE: result = "\t  CHOICE [ ["+obj[1]+"]]: "+str+"\n"+ result; break;
                case SPECIAL: result = "\t  SPECIAL ["+((obj[1].equals(new Character('.')))?"":"\\")+obj[1]+"]: "+str+"\n"+ result; break;
            }
        }

        return result;
    }

    public ArrayList getResults() {
        if ((results == null) || (results.size() == 0))
            return null;
        ArrayList list = new ArrayList();
        for (Iterator iter = results.iterator(); iter.hasNext(); )
            list.add(((ForwardElement)iter.next()).getText());
        return list;
    }

    public class ForwardElement {

        public ForwardElement(int next, String text, String result, ArrayList history) {
            this.next = next;
            this.text = text;
            this.result = result;
            this.history = history;
        }

        private int next;
        private String text;
        private String result = null;
        private ArrayList history;

        public String getResult() {
            return result;
        }

        public void setResult(String result) {
            this.result = result;
        }

        public int getNext() {
            return next;
        }

        public void setHistory(ArrayList list) {
            this.history = list;
        }

        public ArrayList getHistory() {
            return this.history;
        }

        public ArrayList addToHistory(Object[] obj) {
            ArrayList list = new ArrayList(this.history);
            list.add(obj);
            return list;
        }

        public void setNext(int next) {
            this.next = next;
        }

        public String getText() {
            return text;
        }

        public void setText(String text) {
            this.text = text;
        }

    }

}
