/**
 @author Dmitriy I. Suyasov
 @since 24.08.2005 15:20:06
 */
package dsuyasov.inetDoc.rulesZone;

import dsuyasov.inetDoc.functionality.Log;

import java.util.Stack;

public class RuleBuilder {

    private Stack rulesStack = null;
    private String ERROR = "";

    public RuleBuilder() {
        try {
            Rules rules = new Rules(0, 1);
            rulesStack = new Stack();
            rulesStack.push(rules);
        } catch (RuleException re) {
            Log.getLogger().error(re.getMessage());
        }
    }

    public void command(char command, String parameter) throws RuleException {
        Rules rules, r;
        Rule rule;
        switch (command) {
            case '(':
                command('S', parameter.substring(0, parameter.length() - 1));
                r = (Rules) rulesStack.peek();
                rules = new Rules(r.getLastLink(), r.size() + 1);
                rulesStack.push(rules);
                break;
            case ')':
                command('S', parameter.substring(0, parameter.length() - 1));
                rules = (Rules) rulesStack.pop();
                if (rulesStack.isEmpty()) {
                    ERROR = "  :    .";
                    throw new RuleException("  :    .");
                }
                ((Rules) rulesStack.peek()).addRule(rules);
                break;
            case '|':
                command('S', parameter.substring(0, parameter.length() - 1));
                ((Rules) rulesStack.peek()).setLastAsStartNumber();
                break;
            case 'S':
                if ((parameter == null) || (parameter.length() == 0))
                    break;
                r = (Rules) rulesStack.peek();
                rule = new RuleString(parameter, r.getLastLink(), r.size() + 1);
                r.addRule(rule);
                break;
            case '$':
                command('|', parameter+"|");
                break;
            case '\\':
                command('S', parameter.substring(0, parameter.length() - 2));
                if (parameter.charAt(parameter.length() - 1) == '.') {
                    command('S', parameter.substring(parameter.length() - 1));
                    break;
                }
                r = (Rules) rulesStack.peek();
                if (parameter.charAt(parameter.length() - 1) == 'P') {
                    rule = new StringSaver(r.getLastLink(), r.size() + 1);
                } else if (parameter.charAt(parameter.length() - 1) == 'S') {
                    rule = new StringStartSaver(r.getLastLink(), r.size() + 1);
                } else {
                    rule = new RuleSpecial(new Character(parameter.charAt(parameter.length() - 1)), r.getLastLink(), r.size() + 1);
                }
                ((Rules) rulesStack.peek()).addRule(rule);
                break;
            case '.':
                command('S', parameter.substring(0, parameter.length() - 1));
                r = (Rules) rulesStack.peek();
                rule = new RuleSpecial(new Character(parameter.charAt(parameter.length() - 1)), r.getLastLink(), r.size() + 1);
                ((Rules) rulesStack.peek()).addRule(rule);
                break;
            case '[':
                command('S', parameter.substring(0, parameter.length() - 1));
                break;
            case ']':
                r = (Rules) rulesStack.peek();
                rule = new RuleChoice(parameter.substring(0, parameter.length() - 1), r.getLastLink(), r.size() + 1);
                ((Rules) rulesStack.peek()).addRule(rule);
                break;
            case '*':
                if (parameter.length() > 2) command('S', parameter.substring(0, parameter.length() - 2));
                if (parameter.length() > 1) command('S', ""+parameter.charAt(parameter.length() - 2));
                ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(0), null);
                break;
            case '+':
                if (parameter.length() > 2) command('S', parameter.substring(0, parameter.length() - 2));
                if (parameter.length() > 1) command('S', ""+parameter.charAt(parameter.length() - 2));
                ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(1), null);
                break;
            case '?':
                if (parameter.length() > 2) command('S', parameter.substring(0, parameter.length() - 2));
                if (parameter.length() > 1) command('S', ""+parameter.charAt(parameter.length() - 2));
                ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(0), new Integer(1));
                break;
            case '{':
                if (parameter.length() > 2) command('S', parameter.substring(0, parameter.length() - 2));
                if (parameter.length() > 1) command('S', ""+parameter.charAt(parameter.length() - 2));
                break;
            case '}':
                parameter = parameter.substring(0, parameter.length() - 1);
                try {
                    if (parameter.indexOf(',') < 0)
                        ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(parameter), new Integer(parameter));
                    else {
                        int commaPosition = parameter.indexOf(',');
                        if (commaPosition == parameter.length() - 1)
                            ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(parameter.substring(0, commaPosition)), null);
                        else
                            ((Rules) rulesStack.peek()).getLastRule().setRange(new Integer(parameter.substring(0, commaPosition)), new Integer(parameter.substring(commaPosition+1)));
                    }
                } catch (NumberFormatException nfe) {
                    ERROR = "  :     '"+command+"'   ["+parameter+"],       .";
                    throw new RuleException("  :     '"+command+"'   ["+parameter+"],       .");
                }
                break;
            default:
                throw new RuleException("  :    ["+command+"]");
        }
    }

    public String getError() {
        return ERROR;
    }

    public Rule getRules() {
        return (Rule) rulesStack.peek();
    }

}
