/**
 @author Dmitriy I. Suyasov
 @since 01.09.2005 13:31:07
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.functionality.*;
import dsuyasov.inetDoc.main.InetDocErrorConstant;

import java.io.*;
import java.text.MessageFormat;
import java.text.DateFormat;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Date;

public class AutomatAttributeFinder extends Automat {

    public static final int AUTOMAT_NUMBER = 2;

    public static final int STATE_0_READ_FILE = 0;

    public static final int STATE_1_CHECK_HEAD_RULES = 1;

    public static final int STATE_2_READ_HEAD_ATTRIBUTES = 2;

    public static final int STATE_3_CHECK_ENTITY_RULES = 3;

    public static final int STATE_4_TEMPLATE_FILLING = 4;

    public static final int STATE_5_PROCESS_THE_TEMPLATE = 5;

    public static final int STATE_6_PROCESS_HEAD_TEMPLATE = 6;


    /*@
         ,    
     */
    public static final String TOMAP_FILE_NAME = "FILE_NAME";
    /*@
         ,     
     */
    public static final String TOMAP_FILE_ABSOLUTE_NAME = "FILE_ABSOLUTE_NAME";

    public static final String TOMAP_FILE_LAST_MODIFIED_MS = "FILE_LASTMODIFIED_MS";

    public static final String TOMAP_FILE_LAST_MODIFIED = "FILE_LASTMODIFIED";

    public static final String TOMAP_FILE_LENGTH = "FILE_LENGTH";
    /*@
        ,    
    */
    public static final String TOMAP_DOCROOT = "DOCROOT";
    /*@
         ,    
     */
    public static final String TOMAP_SOURCEDIR = "SOURCE_DIR";


    public static final String NAMESPACE = "inetDoc";

    /*@
         
    */
    private Template template;

    private Entity currentEntity;

    /*@
      ,  
    */
    private String pageName = "";

    /*@
      ,  
    */
    private String pageText = "";

//    public String ERROR = "";

    /*@
         
    */
    private int pagePosition = 0;

    private RulesStarter headRulesStarter, entityRulesStarter;

    private MassRulesProcessor massRuleProcessor;
    private AutomatTemplateParser automatTemplateParser;
    private AutomatTemplateFiller automatTemplateFiller;

    private ArrayList templateIterations;

    public Page page;


    public AutomatAttributeFinder() {
        super("dsuyasov.inetDoc.automat.AutomatAttributeFinder");
    }

    public void startAutomat(Template template, String pageName) {
        this.template = template;
        this.pageName = pageName;
        this.pagePosition = 0;
        work();
    }

    public void startAutomat(String pageName) {
        this.template = CommonVariables.getInstance().getTemplateByExtension(pageName);
        if (template == null)
            ERROR = ":      ["+pageName+"]";
        else {
            this.pageName = pageName;
            this.pagePosition = 0;
            work();
        }
    }

    /*@
     
    */
    public void initialize() {
        z0_readFile();
    }

    /*@
      
     @param event   
     */
    public void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);

        switch (y_state) {
            case STATE_0_READ_FILE:
                if (x0_errorFound()) {
                    z19_closeAutomat();
                } else if (x1_fileEndReached()) {
                    z14_initializeTemplateFilling();
                    y_state = STATE_4_TEMPLATE_FILLING;
                    log.automatChangeState(automatName, 0, 4);
                } else {
                    y_state = STATE_1_CHECK_HEAD_RULES;
                    z1_initializeRulesChecking();
                    z2_getNewSymbol();
                    massRuleProcessor.startAutomat(headRulesStarter);
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_CHECK_HEAD_RULES:
                if (x4_isHeadRuleNamedLikeEntity()) {
                    z6_closeAllEntitiesRules();
                    y_state = STATE_2_READ_HEAD_ATTRIBUTES;
                    log.automatChangeState(automatName, 1, 2);
                } else if (x2_thereAreSuccessedRules()) {
                    if (x3_nextSuccessedRuleIsParent()) {
                        z3_changeHeadRuleLinkToParent();
                        z5_killSuccessedRule();
                    } else {
                        z4_changeHeadRuleLinkToChild();
                        z5_killSuccessedRule();
                    }
                } else {
                    y_state = STATE_3_CHECK_ENTITY_RULES;
                    log.automatChangeState(automatName, 1, 3);
                }
                break;
            case STATE_2_READ_HEAD_ATTRIBUTES:
                if (x5_isEntityLikeNamedRuleLeaved()) {
                    y_state = STATE_1_CHECK_HEAD_RULES;
                    log.automatChangeState(automatName, 1, 2);
                } else if (x2_thereAreSuccessedRules()) {
                    if (x3_nextSuccessedRuleIsParent()) {
                        if (x6_currentHeadRuleIsAttributeValueRule()) {
                            z9_saveHeadRuleRusult();
                            z3_changeHeadRuleLinkToParent();
                            z10_addPrefixToEntityRules();
                            z5_killSuccessedRule();
                        } else {
                            z3_changeHeadRuleLinkToParent();
                            z10_addPrefixToEntityRules();
                            z5_killSuccessedRule();
                        }
                    } else {
                        z4_changeHeadRuleLinkToChild();
                        z5_killSuccessedRule();
                    }
                } else if (x1_fileEndReached()) {
                    y_state = STATE_4_TEMPLATE_FILLING;
                    z14_initializeTemplateFilling();
                    log.automatChangeState(automatName, 2, 4);
                } else {
                    z8_addNewSymbolToHeadOnly();
                    massRuleProcessor.startAutomat(headRulesStarter);
                }
                break;
            case STATE_3_CHECK_ENTITY_RULES:
                if (x2_thereAreSuccessedRules()) {
                    if (x3_nextSuccessedRuleIsParent()) {
                        if (x7_currentEntityRuleIsAttributeValueRule()) {
                            z13_saveEntityRuleRusult();
                            z11_changeEntityRuleLinkToParent();
//                            z10_addPrefixToEntityRules();
                            z5_killSuccessedRule();
                        } else {
                            z11_changeEntityRuleLinkToParent();
//                            z10_addPrefixToEntityRules();
                            z5_killSuccessedRule();
                        }
                    } else {
                        z12_changeEntityRuleLinkToChild();
                        z5_killSuccessedRule();
                    }
                } else if (x1_fileEndReached()) {
                    z6_closeAllEntitiesRules();
                    z14_initializeTemplateFilling();
                    y_state = STATE_4_TEMPLATE_FILLING;
                    log.automatChangeState(automatName, 3, 4);
                } else {
                    z2_getNewSymbol();
                    massRuleProcessor.startAutomat(headRulesStarter);
                    y_state = STATE_1_CHECK_HEAD_RULES;
                    log.automatChangeState(automatName, 3, 1);
                }
                break;
            case STATE_4_TEMPLATE_FILLING:
                if (x8_notAllBlocsProcessed()) {
                    y_state = STATE_5_PROCESS_THE_TEMPLATE;
                    log.automatChangeState(automatName, 4, 5);
                } else {
                    y_state = STATE_6_PROCESS_HEAD_TEMPLATE;
                    log.automatChangeState(automatName, 4, 6);
                }
                break;
            case STATE_5_PROCESS_THE_TEMPLATE:
                if (x9_automatsHasErrors()) {
                    z15_saveAutomatError();
                    z19_closeAutomat();
                } else {
                    z16_saveAutomatResult();
                    y_state = STATE_4_TEMPLATE_FILLING;
                    log.automatChangeState(automatName, 5, 4);
                }
                break;
            case STATE_6_PROCESS_HEAD_TEMPLATE:
                if (x9_automatsHasErrors()) {
                    z15_saveAutomatError();
                    z19_closeAutomat();
                } else {
                    z17_saveResultToFileAndToPage();
                    z19_closeAutomat();
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_0_READ_FILE:
                    z0_readFile();
                    break;
                case STATE_1_CHECK_HEAD_RULES:
                    break;
                case STATE_2_READ_HEAD_ATTRIBUTES:
                    z7_setNewEntitiesRules();
                    break;
                case STATE_3_CHECK_ENTITY_RULES:
                    massRuleProcessor.startAutomat(entityRulesStarter);
                    break;
                case STATE_4_TEMPLATE_FILLING:
                    z18_saveFileAndBlocsInformationIntoPage();
                    break;
                case STATE_5_PROCESS_THE_TEMPLATE:
                    automatTemplateParser.startAutomat(((TemplateIteration) templateIterations.get(0)).getTemplateName(), NAMESPACE);
                    automatTemplateFiller.startAutomat(automatTemplateParser.document, new PageDepository(page.setDefaultBloc(((TemplateIteration) templateIterations.get(0)).getBlocNumber())));
                    break;
                case STATE_6_PROCESS_HEAD_TEMPLATE:
                    automatTemplateParser.startAutomat(template.getTemplateFileName(), NAMESPACE);
                    automatTemplateFiller.startAutomat(automatTemplateParser.document, new PageDepository(page.setDefaultBloc(page.getIndexOfBlocWithName(template.getDefaultEntity().getName()))));
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

/*@
      
    @return   
*/
    private boolean x0_errorFound() {
        boolean result = ERROR.length() > 0;
        log.inputVariable(automatName, 0, result);
        return result;
    }

/*@
    ,    
    @return    
*/
    private boolean x1_fileEndReached() {
        boolean result = pageText.length() <= pagePosition;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_thereAreSuccessedRules() {
        boolean result = massRuleProcessor.rulesSuccessed.size() > 0;
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_nextSuccessedRuleIsParent() {
        Object[] obj = (Object[]) massRuleProcessor.rulesSuccessed.get(0);
        boolean result = obj[0].equals(MassRulesProcessor.PARENT);
        log.inputVariable(automatName, 3, result);
        return result;
    }

    private boolean x4_isHeadRuleNamedLikeEntity() {
        boolean result = false;
        if (headRulesStarter.parent != null)
            if (template.getEntity(headRulesStarter.parent.getName()) != null)
                result = true;
        log.inputVariable(automatName, 4, result);
        return result;
    }

    private boolean x5_isEntityLikeNamedRuleLeaved() {
        boolean result = false;
        for (Iterator iter = headRulesStarter.rules.iterator(); iter.hasNext(); )
            if (((RuleStarter) iter.next()).getName().equals(currentEntity.getName()))
                result = true;
        log.inputVariable(automatName, 5, result);
        return result;
    }

    private boolean x6_currentHeadRuleIsAttributeValueRule() {
        String ruleName = headRulesStarter.parent.getName();
        boolean result = currentEntity.hasAttribute(ruleName);
        log.inputVariable(automatName, 6, result);
        return result;
    }

    private boolean x7_currentEntityRuleIsAttributeValueRule() {
        String ruleName = entityRulesStarter.parent.getName();
        boolean result = currentEntity.hasAttribute(ruleName);
        log.inputVariable(automatName, 7, result);
        return result;
    }

    private boolean x8_notAllBlocsProcessed() {
        boolean result = templateIterations.size() > 0;
        log.inputVariable(automatName, 8, result);
        return result;
    }

    private boolean x9_automatsHasErrors() {
        boolean result = (automatTemplateParser.ERROR.length() > 0) || (automatTemplateFiller.ERROR.length() > 0);
        log.inputVariable(automatName, 9, result);
        return result;
    }

//  

    private void z0_readFile() {
        log.output(automatName, 0);
        File file = new File(this.pageName);
        FileReader fr;
        char charbuff[] = new char [(int) file.length()];
        try {
            fr = new FileReader(file);
            fr.read(charbuff);
            pageText = new String(charbuff);
        } catch (FileNotFoundException fnfe) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_FILE_NOT_FOUND + " Stack trace: "+fnfe.getMessage(), new String[] {file.getAbsolutePath()});
        } catch (IOException ioe) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_WHILE_READ_THE_FILE + " Stack trace: "+ioe.getMessage(), new String[] {file.getAbsolutePath()});
        }
    }

    private void z1_initializeRulesChecking() {
        log.output(automatName, 1);
        headRulesStarter = template.getRulesStarter();
        currentEntity = template.getDefaultEntity();
        entityRulesStarter = currentEntity.getRulesStarter();
        pagePosition = 0;
        massRuleProcessor = new MassRulesProcessor();
        page = new Page();
        page.addBloc(new Bloc(currentEntity));
        automatTemplateParser = new AutomatTemplateParser();
        automatTemplateFiller = new AutomatTemplateFiller();
        ERROR = "";
    }

    private void z2_getNewSymbol() {
        log.output(automatName, 2);
        char ch = pageText.charAt(pagePosition);
        headRulesStarter.addCharToStringStack(ch);
        if (entityRulesStarter != null)
            entityRulesStarter.addCharToStringStack(ch);
        pagePosition++;
    }

    private void z3_changeHeadRuleLinkToParent() {
        log.output(automatName, 3);
        headRulesStarter.clearStringStack();
        headRulesStarter = headRulesStarter.parent.getParent();
    }

    private void z4_changeHeadRuleLinkToChild() {
        log.output(automatName, 4);
        Object[] obj = (Object[]) massRuleProcessor.rulesSuccessed.get(0);
        headRulesStarter = (RulesStarter) obj[1];
        headRulesStarter.setStringPrefix((String) obj[2]);
    }

    private void z5_killSuccessedRule() {
        log.output(automatName, 5);
        massRuleProcessor.rulesSuccessed.remove(0);
    }

    private void z6_closeAllEntitiesRules() {
        log.output(automatName, 6);
        if (entityRulesStarter != null) {
            while (entityRulesStarter.parent != null) {
                if (currentEntity.hasAttribute(entityRulesStarter.parent.getName())) {
                    String str = entityRulesStarter.getStringStack();//resultString;
//                    int size = headRulesStarter.resultString.length();
//                    if (size < str.length())
//                        str = str.substring(0, str.length() - size);
                    page.getLastBloc().addAttribute(entityRulesStarter.parent.getName(), str);
                }
                entityRulesStarter.clearStringStack();
                entityRulesStarter = entityRulesStarter.parent.getParent();
            }
            entityRulesStarter.clearStringStack();
        }
    }

    private void z7_setNewEntitiesRules() {
        log.output(automatName, 7);
        //   
        currentEntity = template.getEntity(headRulesStarter.parent.getName());
        //    
        entityRulesStarter = currentEntity.getRulesStarter();
        //  
        page.addBloc(new Bloc(currentEntity));
    }

    private void z8_addNewSymbolToHeadOnly() {
        log.output(automatName, 8);
        char ch = pageText.charAt(pagePosition);
        headRulesStarter.addCharToStringStack(ch);
        pagePosition++;
    }

    private void z9_saveHeadRuleRusult() {
        log.output(automatName, 9);
        Object[] obj = (Object[]) massRuleProcessor.rulesSuccessed.get(0);
        String str = (String) obj[2];
        String ruleResult = headRulesStarter.getStringStack();
        ruleResult = ruleResult.substring(0, ruleResult.length() - str.length());
        page.getLastBloc().addAttribute(headRulesStarter.parent.getName(), ruleResult);
    }

    private void z10_addPrefixToEntityRules() {
        log.output(automatName, 10);
        entityRulesStarter.setStringPrefix((String) ((Object[]) massRuleProcessor.rulesSuccessed.get(0))[2]);
    }

    private void z11_changeEntityRuleLinkToParent() {
        log.output(automatName, 11);
        entityRulesStarter.clearStringStack();
        entityRulesStarter = entityRulesStarter.parent.getParent();
    }

    private void z12_changeEntityRuleLinkToChild() {
        log.output(automatName, 12);
        Object[] obj = (Object[]) massRuleProcessor.rulesSuccessed.get(0);
        entityRulesStarter = (RulesStarter) obj[1];
        entityRulesStarter.setStringPrefix((String) obj[2]);
    }

    private void z13_saveEntityRuleRusult() {
        log.output(automatName, 13);
        Object[] obj = (Object[]) massRuleProcessor.rulesSuccessed.get(0);
        String str = (String) obj[2];
        String ruleResult = entityRulesStarter.getStringStack();
        ruleResult = ruleResult.substring(0, ruleResult.length() - str.length());
        page.getLastBloc().addAttribute(entityRulesStarter.parent.getName(), ruleResult);
    }

    private void z14_initializeTemplateFilling() {
        log.output(automatName, 14);
        templateIterations = new ArrayList();
        for (int iLoop = 0; iLoop < page.blocSize(); iLoop++)
            for (Iterator iter = page.getBloc(iLoop).getEntity().getTemplateFiles().iterator(); iter.hasNext(); )
                templateIterations.add(new TemplateIteration(iLoop, (String) iter.next()));
    }

    private void z15_saveAutomatError() {
        log.output(automatName, 15);
        ERROR = automatTemplateParser.ERROR;
        if (ERROR.length() == 0)
            ERROR = automatTemplateFiller.ERROR;
    }

    private void z16_saveAutomatResult() {
        log.output(automatName, 16);
        page.getBloc(((TemplateIteration) templateIterations.get(0)).getBlocNumber()).setVar("BLOC_TEXT", automatTemplateFiller.resultText);
        templateIterations.remove(0);
    }

    private void z17_saveResultToFileAndToPage() {
        log.output(automatName, 17);

        CommonVariables cv = CommonVariables.getInstance();
        String dirString = pageName.substring(cv.getSourceDirectory().length());
        String fileName = CommonVariables.getInstance().getDestinationDirectory() + dirString + ".html";
        String tmpStr = fileName;
        String directoryName = "";
        while (tmpStr.indexOf(File.separator) >= 0) {
            directoryName += (directoryName.length()>0)?File.separator:"";
            directoryName += tmpStr.substring(0, tmpStr.indexOf(File.separator));
            tmpStr = tmpStr.substring(tmpStr.indexOf(File.separator) + File.separator.length());
            File dir = new File(directoryName);
            dir.mkdir();
        }
        File file = new File(fileName);

        try {
            file.createNewFile();
            FileWriter fw = new FileWriter(file);
            fw.write(automatTemplateFiller.resultText);
            fw.close();
        } catch (IOException ioe) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_CAN_NOT_CREATE_FILE, new String[] {dirString});
        }
    }

    private void z18_saveFileAndBlocsInformationIntoPage() {
        log.output(automatName, 18);

        CommonVariables cv = CommonVariables.getInstance();
        String dirString = pageName.substring(cv.getSourceDirectory().length());
        dirString = (dirString.indexOf(File.separator) == 0)?dirString.substring(File.separator.length()):dirString;
        String sourceDir = cv.getSourceDirectory();
        sourceDir = (sourceDir.lastIndexOf(File.separator) == sourceDir.length() - File.separator.length())?sourceDir.substring(0, sourceDir.length() - File.separator.length()):sourceDir;
        String destDir = CommonVariables.getInstance().getDestinationDirectory();
        destDir = (destDir.lastIndexOf(File.separator) == destDir.length() - File.separator.length())?destDir.substring(0, destDir.length() - File.separator.length()):destDir;
        File file = new File(this.pageName);
        for (int iLoop = 0; iLoop < page.blocSize(); iLoop++) {
            Bloc bloc = page.getBloc(iLoop);
            bloc.setVar("BLOC_TYPE", bloc.getEntity().getName());
            bloc.setVar(TOMAP_FILE_ABSOLUTE_NAME, dirString);
            bloc.setVar(TOMAP_FILE_NAME, (dirString.indexOf(File.separator) >= 0)?dirString.substring(dirString.lastIndexOf(File.separator)+1):dirString);
            bloc.setVar(TOMAP_FILE_LENGTH, ""+file.length());
            bloc.setVar(TOMAP_FILE_LAST_MODIFIED_MS, ""+file.lastModified());
            bloc.setVar(TOMAP_FILE_LAST_MODIFIED, DateFormat.getDateInstance().format(new Date(file.lastModified())));
            bloc.setVar(TOMAP_SOURCEDIR, (sourceDir.lastIndexOf(File.separator) >= 0)?sourceDir.substring(sourceDir.lastIndexOf(File.separator) + 1):sourceDir);
            bloc.setVar(TOMAP_DOCROOT, (destDir.lastIndexOf(File.separator) >= 0)?destDir.substring(destDir.lastIndexOf(File.separator) + 1):destDir);
        }
    }

    private void z19_closeAutomat() {
        log.output(automatName, 19);
        running = false;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "  ";
            case 2: return "  ";
            case 3: return "      ";
            case 4: return "      ";
            case 5: return "     ";
            case 6: return "    ";
            case 7: return "    ";
            case 8: return "     ";
            case 9: return "   ";
            case 10: return "    ";
            case 11: return "      ";
            case 12: return "      ";
            case 13: return "   ";
            case 14: return "    ";
            case 15: return "  ";
            case 16: return "   ";
            case 17: return "  ";
            case 18: return "         ";
            case 19: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "   ";
            case 1: return " ";
            case 2: return "   ";
            case 3: return "   -   ";
            case 4: return "        ";
            case 5: return " ,   ";
            case 6: return "      ";
            case 7: return "      ";
            case 8: return "     ";
            case 9: return "  ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "  ";
            case 2: return "  ";
            case 3: return "  ";
            case 4: return " ";
            case 5: return "    ";
            case 6: return "  ";
        }
        return "    ["+number.intValue()+"]";
    }

}
