/*@java
 @Created by Dmitriy I. Suyasov
 @Date 18.05.2005
 @Time 11:32:48
 */
package dsuyasov.inetDoc.functionality;

import org.w3c.dom.Node;

import java.util.*;

import dsuyasov.inetDoc.rulesZone.Rule;
import dsuyasov.inetDoc.automat.AutomatSintaxRuleAnalyser;

public class RuleStarter {

    /*@
          -  
    */
    public static final String START_NODE_NAME = "start";
    /*@
          -  
    */
    public static final String END_NODE_NAME = "end";

    public static final String PRIORITY_ATTRIBUTE = "priority";

    public static final String START_PRIORITY_ATTRIBUTE = "startpriority";

    public static final String END_PRIORITY_ATTRIBUTE = "endpriority";

    public static final String NAME_ATTRIBUTE = "name";

    /*@
         
    */
    private String name;

    private ArrayList id;

    /*@
          
    */
    private RulesStarter rules;
    /*@
         - 
    */
    private RulesStarter parent;

    private Rule startRule = null, endRule = null;

    private Integer priority = null;
    private Integer endPriority = null;
    private Integer startPriority = null;

    /*@
    .    XML .
    @param xml    XML 
    @param node    
    @param defines    -
    @param parent   -
    */
    public RuleStarter(Node node, RulesStarter parent, ArrayList id) throws Exception {

        this.parent = parent;
        this.id = id;

        try {
            AutomatSintaxRuleAnalyser automat;

            automat = new AutomatSintaxRuleAnalyser();
            automat.startAutomat(getStartRegularExtention(node));
            if (!automat.ERROR.equals(""))
                throw new Exception(automat.ERROR);
            startRule = automat.ruleBuilder.getRules();

            automat = new AutomatSintaxRuleAnalyser();
            automat.startAutomat(getEndRegularExtention(node));
            if (!automat.ERROR.equals(""))
                throw new Exception(automat.ERROR);
            endRule = automat.ruleBuilder.getRules();

            rules = new RulesStarter(node, this);

            name = XMLProcessor.getNodeAttribute(node, NAME_ATTRIBUTE);

            try {
                priority = new Integer(XMLProcessor.getNodeAttribute(node, PRIORITY_ATTRIBUTE));
            } catch (XMLProcessorException xmlpe) {
                priority = null;
            }
            try {
                startPriority = new Integer(XMLProcessor.getNodeAttribute(node, START_PRIORITY_ATTRIBUTE));
            } catch (XMLProcessorException xmlpe) {
                startPriority = null;
            }
            try {
                endPriority = new Integer(XMLProcessor.getNodeAttribute(node, END_PRIORITY_ATTRIBUTE));
            } catch (XMLProcessorException xmlpe) {
                endPriority = null;
            }
        } catch (Exception e) {
            throw new Exception("   xml- ->\n"+e.getMessage());
        }
    }

    private String getStartRegularExtention(Node node) {
        Node startNode = XMLProcessor.getNode(node, START_NODE_NAME, 1);
        return getRegularExtention(startNode);
    }

    private String getEndRegularExtention(Node node) {
        Node endNode = XMLProcessor.getNode(node, END_NODE_NAME, 1);
        return getRegularExtention(endNode);
    }

    private String getRegularExtention(Node node) {
        return XMLProcessor.getNodeValue(node);
    }

    public Integer getPriority() {
        return priority;
    }

    public Integer getStartPriority() {
        return startPriority;
    }

    public Integer getEndPriority() {
        return endPriority;
    }

    public ArrayList getId() {
        return id;
    }

    public RuleStarter findById(String id) {
        RulesStarter rs = getRulesStarter();
        while (rs.parent != null)
            rs = rs.parent.getParent();
        return getStarterById(rs, id);
    }

    private RuleStarter getStarterById(RulesStarter rs, String id) {
        for (Iterator iter = rs.rules.iterator(); iter.hasNext(); ) {
            RuleStarter starter = (RuleStarter) iter.next();
            if (starter.getId().equals(id))
                return starter;
            starter = getStarterById(starter.getRulesStarter(), id);
            if (starter != null)
                return starter;
        }
        return null;
    }

    /*@
             
        @param name  
        @return  
    */
    public ArrayList getRule(String name) {
        ArrayList toReturn = new ArrayList();
        if (this.name.equals(name))
            toReturn.add(this);
        for (int i = 0; i < rules.rules.size(); i++)
            toReturn.addAll(((RuleStarter)rules.rules.get(i)).getRule(name));
        return toReturn;
    }

    /*@
          
        @return  
    */
    public String getName() {
        return name;
    }

    /*@
         -
        @return 
    */
    public RulesStarter getParent() {
        return this.parent;
    }

    /*@
          
        @return  
    */
    public ArrayList getAllRules() {
        return rules.rules;
    }

    public RulesStarter getRulesStarter() {
        return rules;
    }

    public Object processStart() {
        Boolean result = (startRule.proceed(rules.getStringStack())) ? Boolean.TRUE : Boolean.FALSE;
        if (result.booleanValue())
            return startRule.getResult();
        return result;
    }

    public Object processEnd() {
        Boolean result = (endRule.proceed(rules.getStringStack())) ? Boolean.TRUE : Boolean.FALSE;
        if (result.booleanValue())
            return endRule.getResult();
        return result;
    }

    public Object processStart(String str) {
        Boolean result = (startRule.proceed(str)) ? Boolean.TRUE : Boolean.FALSE;
        if (result.booleanValue())
            return startRule.getResult();
        return result;
    }

    public Object processEnd(String str) {
        Boolean result = (endRule.proceed(str)) ? Boolean.TRUE : Boolean.FALSE;
        if (result.booleanValue())
            return endRule.getResult();
        return result;
    }

    public String getEndHistory() {
        return endRule.getHistory();
    }

    public String getStartHistory() {
        return startRule.getHistory();
    }

    public String toString() {
        String str = "{\n";
        str += "startRule = "+startRule+",\n";
        str += "endRule = "+endRule+"\n";
        str += " }";
        return str;
    }

}
