/**
 @author Dmitriy Suyasov
 @since 10.10.2005 10:02:34
 */
package dsuyasov.inetDoc.aho;

import dsuyasov.inetDoc.automat.Automat;
import dsuyasov.inetDoc.functionality.Builder;

public class TreeBuilder implements Builder {

    private Tree tree;
    private Automat automat;

    public TreeBuilder() {
        clearTree();
    }

    public Object getResult() {
        return automat;
    }

    public String getError() {
        return "";
    }

    public void clearTree() {
        tree = new Tree();
    }

    public void command(char command, String parameter) throws Exception {
        switch (command) {
            case '.':
            case '|':
            case '[':
            case 'S':
                for (int iLoop = 0; iLoop < parameter.length(); iLoop++) {
                    tree.createAChild().addToText("" + parameter.charAt(iLoop));
                    tree.gotoParent();
                }
                break;
            case '(':
                command('S', parameter.substring(0, parameter.length() - 1));
                tree.createAChild().setText("(");
                break;
            case ')':
                command('S', parameter.substring(0, parameter.length() - 1));
                tree.getCurrentElement().addToText(")");
                tree.gotoParent();
                break;
            case '{':
            case '?':
            case '+':
            case '*':
                command('S', parameter.substring(0, parameter.length() - 1));
                tree.gotoLastChild().addToText(""+command);
                tree.gotoParent();
                break;
            case ']':
            case '}':
                tree.gotoLastChild().addToText(parameter);
                tree.gotoParent();
                break;
            case '\\':
                if (parameter.length() > 2)
                    command('S', parameter.substring(0, parameter.length() - 2));
                tree.createAChild().setText(parameter.substring(parameter.length() - 2));
                tree.gotoParent();
                break;
            case '$':
                command('S', parameter);
                if (!tree.getCurrentElement().getText().equals("("))
                    automat = AutomatBuilder.build(tree);
                break;
        }
    }

}
