/**
 @author Dmitriy I. Suyasov
 @since 08.09.2005 13:10:03
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.functionality.CommonVariables;

public class AutomatArgumentProcessor extends Automat {

    public static final int AUTOMAT_NUMBER = 1;

    public AutomatArgumentProcessor() {
        super("dsuyasov.inetDoc.automat.AutomatArgumentProcessor");
    }

    public static final int STATE_0_START_WORK = 0;
    public static final int STATE_1_PROCESS_COMMAND = 1;
//    public static final int STATE_2_READ_PARAMETER = 2;
    public static final int STATE_2_PROCESS_COMMAND_WITH_PARAMETER = 2;

    //   
    /*@
            -     
     */
    public static final String ARGUMENT_LOG_FILE = "-o";
    /*@
            -     
     */
    public static final String ARGUMENT_HELP = "-h";
    /*@
            -    
     */
    public static final String ARGUMENT_CAN_LOG_TO_FILE = "-l";

    public static final String ARGUMENT_DOCUMENT = "-d";

    public static final String ARGUMENT_PROPERTIES = "-p";

    public static final String ARGUMENT_RULES_LOG_FILE = "-r";
    /*@
           -    
    */
    public static final String ARGUMENT_CAN_LOG_TO_SCREEN = "-s";

    /*@
          
     */
    public static final String EMPTY_PROGRAM_OUT = " : java -jar inetDoc.jar [] <     >\n\n" +
            "    :\n" +
            "-h                            ( )\n" +
            "-l                           (  )\n" +
            "-s                           (  )\n" +
            "-d < >             (documentation   )\n" +
            "-p <  >    ,      (\"inetDoc.xml\"   )\n" +
            "-o <  >     (\"log.txt\"   )\n" +
            "-r <  >       (   )\n\n";

    private String[] arguments;
    private int argumentsIndex;
    private String command;
    private String parameter;

    public void startAutomat(String[] arguments) {
        this.arguments = arguments;
        this.argumentsIndex = 0;
        work();
    }

    /*@
          
    */
    protected void initialize() {}

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_START_WORK:
                if (x0_stringIsEmpty()) {
                    z0_saveError();
                    z11_closeAutomat();
                } else {
                    y_state = STATE_1_PROCESS_COMMAND;
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case STATE_1_PROCESS_COMMAND:
                if (x1_commandIsH()) {
                    z3_logOutHelpStrings();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 1, 0);
                } else if (x2_commandIsS()) {
                    z4_setProgramCanLogToScreen();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 1, 0);
                } else if (x3_commandIsL()) {
                    z5_setProgramCanLogToFile();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 1, 0);
                } else if (x0_stringIsEmpty()) {
                    z9_setSource();
                    z11_closeAutomat();
                } else {
                    y_state = STATE_2_PROCESS_COMMAND_WITH_PARAMETER;
                    log.automatChangeState(automatName, 1, 2);
                }
                break;
            case STATE_2_PROCESS_COMMAND_WITH_PARAMETER:
                if (x4_commandIsO()) {
                    z6_setLogFileName();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 2, 0);
                } else if (x5_commandIsD()) {
                    z7_setDestinationFileName();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 2, 0);
                } else if (x6_commandIsP()) {
                    z8_setPropertyFileName();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 2, 0);
                } else if (x7_commandIsR()) {
                    z10_setRulesLogFile();
                    y_state = STATE_0_START_WORK;
                    log.automatChangeState(automatName, 2, 0);
                } else {
                    z0_saveError();
                    z11_closeAutomat();
                }
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_1_PROCESS_COMMAND:
                    z1_readStringToCommand();
                    break;
                case STATE_2_PROCESS_COMMAND_WITH_PARAMETER:
                    z2_readStringToParameter();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_stringIsEmpty() {
        boolean result = argumentsIndex >= arguments.length;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_commandIsH() {
        boolean result = command.equals(ARGUMENT_HELP);
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_commandIsS() {
        boolean result = command.equals(ARGUMENT_CAN_LOG_TO_SCREEN);
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_commandIsL() {
        boolean result = command.equals(ARGUMENT_CAN_LOG_TO_FILE);
        log.inputVariable(automatName, 3, result);
        return result;
    }

    private boolean x4_commandIsO() {
        boolean result = command.equals(ARGUMENT_LOG_FILE);
        log.inputVariable(automatName, 4, result);
        return result;
    }

    private boolean x5_commandIsD() {
        boolean result = command.equals(ARGUMENT_DOCUMENT);
        log.inputVariable(automatName, 5, result);
        return result;
    }

    private boolean x6_commandIsP() {
        boolean result = command.equals(ARGUMENT_PROPERTIES);
        log.inputVariable(automatName, 6, result);
        return result;
    }

    private boolean x7_commandIsR() {
        boolean result = command.equals(ARGUMENT_RULES_LOG_FILE);
        log.inputVariable(automatName, 7, result);
        return result;
    }

    //  

    private void z0_saveError() {
        log.output(automatName, 0);
        ERROR = ":    \n---------------\n"+EMPTY_PROGRAM_OUT;
    }

    private void z1_readStringToCommand() {
        log.output(automatName, 1);
        command = arguments[argumentsIndex];
        argumentsIndex++;
    }

    private void z2_readStringToParameter() {
        log.output(automatName, 2);
        parameter = arguments[argumentsIndex];
        argumentsIndex++;
    }

    private void z3_logOutHelpStrings() {
        log.output(automatName, 3);
        log.out(EMPTY_PROGRAM_OUT);
    }

    private void z4_setProgramCanLogToScreen() {
        log.output(automatName, 4);
        CommonVariables.getInstance().setLogToScreen(true);
    }

    private void z5_setProgramCanLogToFile() {
        log.output(automatName, 5);
        CommonVariables.getInstance().setLogToFile(true);
    }

    private void z6_setLogFileName() {
        log.output(automatName, 6);
        CommonVariables.getInstance().setLogFileName(parameter);
    }

    private void z7_setDestinationFileName() {
        log.output(automatName, 7);
        CommonVariables.getInstance().setDestinationDirectory(parameter);
    }

    private void z8_setPropertyFileName() {
        log.output(automatName, 8);
        try {
            CommonVariables.getInstance().setPropertyFileName(parameter);
        } catch (Exception e) {
            ERROR = ": "+e.getMessage();
        }
    }

    private void z9_setSource() {
        log.output(automatName, 9);
        try {
            CommonVariables.getInstance().setSourceDirectory(command);
        } catch (RuntimeException re) {
            ERROR = ": "+re.getMessage();
        }
    }

    private void z10_setRulesLogFile() {
        log.output(automatName, 10);
        CommonVariables.getInstance().setRulesLogFile(parameter);
    }

    private void z11_closeAutomat() {
        log.output(automatName, 11);
        running = false;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "   ";
            case 2: return "   ";
            case 3: return "     ";
            case 4: return ",       ";
            case 5: return ",       ";
            case 6: return "   ";
            case 7: return "    ";
            case 8: return "    ";
            case 9: return "    ";
            case 10: return "      ";
            case 11: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "  ["+ARGUMENT_HELP+"]";
            case 2: return "  ["+ARGUMENT_CAN_LOG_TO_SCREEN+"]";
            case 3: return "  ["+ARGUMENT_CAN_LOG_TO_FILE+"]";
            case 4: return "  ["+ARGUMENT_LOG_FILE+"]";
            case 5: return "  ["+ARGUMENT_DOCUMENT+"]";
            case 6: return "  ["+ARGUMENT_PROPERTIES+"]";
            case 7: return "  ["+ARGUMENT_RULES_LOG_FILE+"]";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return " ";
            case 2: return "   ";
        }
        return "    ["+number.intValue()+"]";
    }

}
