/*@java
 @Created by Dmitriy I. Suyasov
 @Date 07.04.2005
 @Time 12:21:53
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.functionality.*;
import dsuyasov.inetDoc.main.InetDocErrorConstant;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.*;
import java.io.*;
import java.text.MessageFormat;
import java.text.DateFormat;

import org.w3c.dom.Document;
import org.w3c.dom.Node;


public class AutomatMain extends Automat{

    public static final int AUTOMAT_NUMBER = 0;

    /*@
        0 -   
    */
    public static final int STATE_0_START_WORK = 0;
    /*@
         1 -  
     */
    public static final int STATE_1_LOAD_PROPERTIES = 1;
    /*@
         2 -  
     */
    public static final int STATE_2_PROCESS = 2;
    /*@
         3 -       
     */
    public static final int STATE_3_PROCESS_FILE_AND_FILL_TEMPLATE = 3;
    /*@
         4 -  
     */
    public static final int STATE_4_FILL_TEMPLATE = 4;
    /*@
         5 -   
     */
    public static final int STATE_5_CREATING_COMMON_FILES = 5;
    /*@
         6 -     
     */
    public static final int STATE_6_FILL_TEMPLATE_FOR_COMMON_FILE = 6;
    /*@
         7 -  
     */
    public static final int STATE_7_END_WORK = 7;

    /*@
        
    */
    private AutomatAttributeFinder automatAttributeFinder;
    private AutomatArgumentProcessor automatArgumentProcessor;
    private AutomatTemplateParser automatTemplateParser;
    private AutomatTemplateFiller automatTemplateFiller;

    private Page page;
    private String[] arguments;

    /*@
       .
       @param automatNumber  
    */
    public AutomatMain() {
        super("dsuyasov.inetDoc.automat.AutomatMain");
        automatArgumentProcessor = new AutomatArgumentProcessor();
        automatAttributeFinder = new AutomatAttributeFinder();
        automatTemplateParser = new AutomatTemplateParser();
        automatTemplateFiller = new AutomatTemplateFiller();
    }

    public void startAutomat(String[] arguments) {
        this.arguments = arguments;
        work();
    }

    /*@
      
    */
    protected void initialize() {
        automatArgumentProcessor.startAutomat(arguments);
    }

    /*@
         
        @param event   
     */
    protected synchronized void automat(int event) {
        int y0_statePrevious = y_state;
        log.addAutomatStartWork(automatName, y_state);

        switch (y_state) {
            case (STATE_0_START_WORK):
                if (x1_automatArgumentProcessorHasError()) {
                    z1_saveAutomatsError();
                    y_state = STATE_7_END_WORK;
                    log.automatChangeState(automatName, 0, 7);
                } else {
                    y_state = STATE_1_LOAD_PROPERTIES;
                    log.automatChangeState(automatName, 0, 1);
                }
                break;
            case (STATE_1_LOAD_PROPERTIES):
                if (x0_errorFound()) {
                    y_state = STATE_7_END_WORK;
                    log.automatChangeState(automatName, 1, 7);
                } else if (x4_enoughSettingsToStartWork()) {
                    y_state = STATE_2_PROCESS;
                    z5_getFilesList();
                    log.automatChangeState(automatName, 1, 2);
                } else {
                    z3_setErrorAsNotEnoughSettings();
                }
                break;
            case (STATE_2_PROCESS):
                if (x0_errorFound()) {
                    y_state = STATE_7_END_WORK;
                    log.automatChangeState(automatName, 2, 7);
                } else if (x6_newFileToDocumentExist()) {
                    if (x7_templateForNextFileExist()) {
                        y_state = STATE_3_PROCESS_FILE_AND_FILL_TEMPLATE;
                        log.automatChangeState(automatName, 2, 3);
                    } else {
                        y_state = STATE_4_FILL_TEMPLATE;
                        z11_createInfoForFillTemplateAutomat();
                        log.automatChangeState(automatName, 2, 4);
                    }
                } else {
                    y_state = STATE_5_CREATING_COMMON_FILES;
                    log.automatChangeState(automatName, 2, 5);
                }
                break;
            case (STATE_3_PROCESS_FILE_AND_FILL_TEMPLATE):
                if  (x3_AttributeFinderAutomatHasErrors()) {
                    y_state = STATE_7_END_WORK;
                    z1_saveAutomatsError();
                    log.automatChangeState(automatName, 3, 7);
                } else {
                    y_state = STATE_2_PROCESS;
                    z8_saveResultPageIntoPagesScope();
                    z6_killTopFileName();
                    log.automatChangeState(automatName, 3, 2);
                }
                break;
            case (STATE_4_FILL_TEMPLATE):
                if (x2_templateAutomatsHasErrors()) {
                    y_state = STATE_7_END_WORK;
                    z1_saveAutomatsError();
                    log.automatChangeState(automatName, 4, 7);
                } else {
                    y_state = STATE_2_PROCESS;
                    z9_saveFillTemplateAutomatResult();
                    z6_killTopFileName();
                    log.automatChangeState(automatName, 4, 2);
                }
                break;
            case STATE_5_CREATING_COMMON_FILES:
                if (x0_errorFound()) {
                    y_state = STATE_7_END_WORK;
                    log.automatChangeState(automatName, 5, 7);
                } else if (x5_haveUnprocessedCommonFile()) {
                    y_state = STATE_6_FILL_TEMPLATE_FOR_COMMON_FILE;
                    log.automatChangeState(automatName, 5, 6);
                } else {
                    y_state = STATE_7_END_WORK;
                    log.automatChangeState(automatName, 5, 7);
                }
                break;
            case STATE_6_FILL_TEMPLATE_FOR_COMMON_FILE:
                if (x2_templateAutomatsHasErrors()) {
                    y_state = STATE_7_END_WORK;
                    z1_saveAutomatsError();
                    log.automatChangeState(automatName, 6, 7);
                } else {
                    y_state = STATE_5_CREATING_COMMON_FILES;
                    z10_saveAutomatResultForCommonFilesToFile();
                    z7_killTopCommonFileName();
                    log.automatChangeState(automatName, 6, 5);
                }
                break;
            case STATE_7_END_WORK:
                if (x0_errorFound()) {
                    z2_logError();
                    z0_closeAutomatAndLogMessagesOut();
                } else {
                    z0_closeAutomatAndLogMessagesOut();
                }
                break;
        }

        if (y0_statePrevious != y_state)
            switch (y_state) {
                case (STATE_1_LOAD_PROPERTIES):
                    z4_loadProperties();
                    break;
                case (STATE_3_PROCESS_FILE_AND_FILL_TEMPLATE):
                    automatAttributeFinder.startAutomat((String) CommonVariables.getInstance().getFiles().get(0));
                    break;
                case (STATE_4_FILL_TEMPLATE):
                    automatTemplateParser.startAutomat(CommonVariables.getInstance().getDefaultTemplate(), AutomatAttributeFinder.NAMESPACE);
                    automatTemplateFiller.startAutomat(automatTemplateParser.document, new PageDepository(page));
                    break;
                case (STATE_6_FILL_TEMPLATE_FOR_COMMON_FILE):
                    automatTemplateParser.startAutomat(((String[]) CommonVariables.getInstance().getCommonFiles().get(0))[1], AutomatAttributeFinder.NAMESPACE);
                    automatTemplateFiller.startAutomat(automatTemplateParser.document, CommonVariables.getInstance().getPageDepository());
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }

    //  

    /*@
            
        @return    
    */
    private boolean x0_errorFound() {
        boolean result = ERROR.length() > 0;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_automatArgumentProcessorHasError() {
        boolean result = automatArgumentProcessor.ERROR.length() > 0;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_templateAutomatsHasErrors() {
        boolean result = (automatTemplateParser.ERROR.length() > 0) || (automatTemplateFiller.ERROR.length() > 0);
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_AttributeFinderAutomatHasErrors() {
        boolean result = automatAttributeFinder.ERROR.length() > 0;
        log.inputVariable(automatName, 3, result);
        return result;
    }

    /*@
             
       @return     
    */
    private boolean x4_enoughSettingsToStartWork() {
        boolean result = CommonVariables.getInstance().getSourceDirectory().length() > 0;
        log.inputVariable(automatName, 4, result);
        return result;
    }

    /*@
           
       @return     
    */
    private boolean x5_haveUnprocessedCommonFile() {
        CommonVariables cv = CommonVariables.getInstance();
        boolean result = cv.getCommonFiles().size() > 0;
        log.inputVariable(automatName, 5, result);
        return result;
    }

    /*@
          
       @return    
    */
    private boolean x6_newFileToDocumentExist() {
        CommonVariables cv = CommonVariables.getInstance();
        boolean result = cv.getFiles().size() > 0;
        log.inputVariable(automatName, 6, result);
        return result;
    }

    /*@
             
       @return      
    */
    private boolean x7_templateForNextFileExist() {
        CommonVariables cv = CommonVariables.getInstance();
        String pageFileName = (String) cv.getFiles().get(0);
        for (int iLoop = 0; iLoop < cv.getTemplates().size(); iLoop++) {
            ArrayList types = ((Template) cv.getTemplates().get(iLoop)).getTypes();
            for (int iLoop2 = 0; iLoop2 < types.size(); iLoop2++)
                if (pageFileName.lastIndexOf((String) types.get(iLoop2)) == pageFileName.length() - ((String) types.get(iLoop2)).length()) {
                    log.inputVariable(automatName, 7, true);
                    return true;
                }
        }
        log.inputVariable(automatName, 7, false);
        return false;
    }


    //  

    /*@
         
    */
    private void z0_closeAutomatAndLogMessagesOut() {
        log.output(automatName, 0);
        running = false;

        log.addAutomatFinishWork(automatName, y_state);
        log.logOutAllMessages();
    }

    /*@
          
    */
    private void z1_saveAutomatsError() {
        log.output(automatName, 1);
        ERROR = automatArgumentProcessor.ERROR;
        if (ERROR.length() == 0)
            ERROR = automatAttributeFinder.ERROR;
        if (ERROR.length() == 0)
            ERROR = automatTemplateParser.ERROR;
        if (ERROR.length() == 0)
            ERROR = automatTemplateFiller.ERROR;
    }

    /*@
               
    */
    private void z2_logError() {
        log.output(automatName, 2);
        log.error("["+automatName+"] ->\n"+ERROR);
    }

    private void z3_setErrorAsNotEnoughSettings() {
        log.output(automatName, 3);
        ERROR = ":     ";
    }

    /*@
           
    */
    private void z4_loadProperties() {
        log.output(automatName, 4);
        CommonVariables cv = CommonVariables.getInstance();
        try {
            DocumentBuilder db =  DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = db.parse(CommonVariables.getInstance().getPropertyFileName());
            Node mainNode = XMLProcessor.getNode(doc, "inetdoc");
            cv.setDefaultTemplate(XMLProcessor.getNodeAttribute(XMLProcessor.getNode(mainNode, "default"), "template"));
            ArrayList templates = new ArrayList();
            ArrayList commonFiles = new ArrayList();

            Collection nodes = XMLProcessor.getNodes(mainNode, "common");
            for (Iterator iter = nodes.iterator(); iter.hasNext(); ) {
                Node node = (Node) iter.next();
                String name = XMLProcessor.getNodeAttribute(node, "name");
                String tmp = XMLProcessor.getNodeAttribute(node, "template");
                commonFiles.add(new String[] {name, tmp});
            }
            cv.setCommonFiles(commonFiles);

            nodes = XMLProcessor.getNodes(XMLProcessor.getNodes(mainNode, "filesrecognition"), "file");
            for (Iterator iter = nodes.iterator(); iter.hasNext(); ) {
                Node node = (Node) iter.next();
                ArrayList types = new ArrayList(XMLProcessor.getNodesAttribute(XMLProcessor.getNodes(node, "extension"), "name"));
                if (types.size() == 0)
                    throw new Exception("   () ()");
                templates.add(new Template(types, XMLProcessor.getNodeAttribute(XMLProcessor.getNode(node, "template"), "href")));
            }
            cv.setTemplates(templates);
        } catch (Exception e) {
            ERROR = /*MessageFormat.format(InetDocErrorConstant.ERROR_PROCESS_PROPERTY, new String[] {cv.getPropertyFileName()}) + */e.getMessage();
        }
    }

    /*@
         ,  
    */
    private void z5_getFilesList() {
        log.output(automatName, 5);
        CommonVariables cv = CommonVariables.getInstance();
        ArrayList files = new ArrayList();

        try {
            cv.getAllFiles(files, cv.getSourceDirectory());
        } catch (Exception e) {
            ERROR = InetDocErrorConstant.ERROR_GET_FILE_NAMES;
        }
        cv.setFiles(files);
    }

    private void z6_killTopFileName() {
        log.output(automatName, 6);
        CommonVariables.getInstance().getFiles().remove(0);
    }

    private void z7_killTopCommonFileName() {
        log.output(automatName, 7);
        CommonVariables.getInstance().getCommonFiles().remove(0);
    }

    private void z8_saveResultPageIntoPagesScope() {
        log.output(automatName, 8);
        CommonVariables.getInstance().saveToPageDepository(automatAttributeFinder.page);
    }

    /*@
               
    */
    private void z9_saveFillTemplateAutomatResult() {
        log.output(automatName, 9);
        CommonVariables cv = CommonVariables.getInstance();

        cv.saveToPageDepository(page);

        //      
        String resultText = automatTemplateFiller.resultText;
        String name = cv.getDestinationDirectory() + ((String) cv.getFiles().get(0)).substring(cv.getSourceDirectory().length()) + ".html";

        //  ,      
        String tmpStr = name;
        String directoryName = "";
        while (tmpStr.indexOf(File.separator) >= 0) {
            directoryName += (directoryName.length()>0)?File.separator:"";
            directoryName += tmpStr.substring(0, tmpStr.indexOf(File.separator));
            tmpStr = tmpStr.substring(tmpStr.indexOf(File.separator) + File.separator.length());
            File dir = new File(directoryName);
            dir.mkdir();
        }

        //   
        File file = new File(name);
        try {
            FileWriter fw = new FileWriter(file);
            fw.write(resultText);
            fw.close();
        }
        catch (Exception e) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_WHILE_WRITE_THE_FILE + " Stack trace: "+e.getMessage(), new String[] {file.getAbsolutePath()});
        }
    }

    /*@
                
    */
    private void z10_saveAutomatResultForCommonFilesToFile() {
        log.output(automatName, 10);
        CommonVariables cv = CommonVariables.getInstance();
        String name = ((String[]) cv.getCommonFiles().get(0))[0];
        String fileText = automatTemplateFiller.resultText;

        String nameprefix = "";
        if (name.indexOf(File.separator) != 0)
            nameprefix = File.separator;
        nameprefix = cv.getDestinationDirectory() + nameprefix;

        String tmpStr = name;
        String directoryName = "";
        while (tmpStr.indexOf(File.separator) >= 0) {
            directoryName += (directoryName.length()>0)?File.separator:"";
            directoryName += tmpStr.substring(0, tmpStr.indexOf(File.separator));
            tmpStr = tmpStr.substring(tmpStr.indexOf(File.separator) + File.separator.length());
            File dir = new File(nameprefix + directoryName);
            dir.mkdir();
        }

        File file = new File(nameprefix + name);
        try {
            FileWriter fw = new FileWriter(file);
            fw.write(fileText);
            fw.close();
        }
        catch (Exception e) {
            ERROR = MessageFormat.format(InetDocErrorConstant.ERROR_WHILE_WRITE_THE_FILE + " Stack trace: "+e.getMessage(), new String[] {file.getAbsolutePath()});
        }
    }

    /*@
                  
    */
    private void z11_createInfoForFillTemplateAutomat() {
        log.output(automatName, 11);
        CommonVariables cv = CommonVariables.getInstance();

        //       
        page = new Page();
        Entity entity = null;
        Bloc bloc = new Bloc(entity);

        String dirString = ((String) cv.getFiles().get(0)).substring(cv.getSourceDirectory().length());
        String sourceDir = cv.getSourceDirectory();
        sourceDir = (sourceDir.lastIndexOf(File.separator) == sourceDir.length() - File.separator.length())?sourceDir.substring(0, sourceDir.length() - File.separator.length()):sourceDir;
        String destDir = CommonVariables.getInstance().getDestinationDirectory();
        destDir = (destDir.lastIndexOf(File.separator) == destDir.length() - File.separator.length())?destDir.substring(0, destDir.length() - File.separator.length()):destDir;
        File file = new File((String) cv.getFiles().get(0));

        bloc.setVar(AutomatAttributeFinder.TOMAP_FILE_ABSOLUTE_NAME, dirString);
        bloc.setVar(AutomatAttributeFinder.TOMAP_FILE_NAME, (dirString.indexOf(File.separator) >= 0)?dirString.substring(dirString.lastIndexOf(File.separator)+1):dirString);
        bloc.setVar(AutomatAttributeFinder.TOMAP_FILE_LENGTH, ""+file.length());
        bloc.setVar(AutomatAttributeFinder.TOMAP_FILE_LAST_MODIFIED_MS, ""+file.lastModified());
        bloc.setVar(AutomatAttributeFinder.TOMAP_FILE_LAST_MODIFIED, DateFormat.getDateInstance().format(new Date(file.lastModified())));
        bloc.setVar(AutomatAttributeFinder.TOMAP_SOURCEDIR, (sourceDir.lastIndexOf(File.separator) >= 0)?sourceDir.substring(sourceDir.lastIndexOf(File.separator) + 1):sourceDir);
        bloc.setVar(AutomatAttributeFinder.TOMAP_DOCROOT, (destDir.lastIndexOf(File.separator) >= 0)?destDir.substring(destDir.lastIndexOf(File.separator) + 1):destDir);
        page.addBloc(bloc);
        page.setDefault(0);
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "     ";
            case 1: return "  - ";
            case 2: return "   ";
            case 3: return "  -     ";
            case 4: return "     ";
            case 5: return "    ";
            case 6: return "     ";
            case 7: return "      ";
            case 8: return "       ";
            case 9: return "     ";
            case 10: return "        ";
            case 11: return "       c  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "   ";
            case 1: return "-    ";
            case 2: return "       ";
            case 3: return "     ";
            case 4: return "    ";
            case 5: return "    ";
            case 6: return "   ";
            case 7: return " -   ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return " ";
            case 2: return " ";
            case 3: return "      ";
            case 4: return " ";
            case 5: return "  ";
            case 6: return "    ";
            case 7: return " ";
        }
        return "    ["+number.intValue()+"]";
    }

}