/**
 @author Dmitriy I. Suyasov
 @since 24.08.2005 16:32:39
 */
package dsuyasov.inetDoc.automat;

import dsuyasov.inetDoc.rulesZone.RuleBuilder;
import dsuyasov.inetDoc.functionality.Builder;
import dsuyasov.inetDoc.aho.TreeBuilder;

import java.util.Stack;

public class AutomatSintaxRuleAnalyser extends Automat {

    public static final int AUTOMAT_NUMBER = 6;

    public static final int STATE_0_STACK_ANALYSIS = 0;
    public static final int STATE_1_TERMINAL = 1;
    public static final int STATE_2_NONTERMINAL = 2;
    public static final int STATE_3_ERROR = 3;

    public Builder ruleBuilder;
    private AutomatLexicalRuleAnalyser lexicalAnalyser;

    private Stack stack = new Stack();
    private String formula = "";
    private String expression = "";
    private String regularExpression;
    private boolean byAho;

    private final String terminals = "().[]+*?|{},-D\\C^$";

    private final String nonterminals = "SsELlMKkNnFPpUu";

    //                                (   )   .   [   ]   +   *   ?   |   {   }   ,   -   D   \   C   ^   $
    private int[][] table = {/* S */{ 1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1,  1},
                             /* s */{ 0,  2,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  0,  0,  0,  0,  2},
                             /* E */{ 4, 56,  4,  4,  0,  0,  0,  0, 56,  0,  0,  4,  4,  4,  4,  4,  4, 56},
                             /* L */{ 8,  0, 10,  5,  0,  0,  0,  0,  0,  0,  0, 54, 11,  9, 13,  9, 12,  0},
                             /* l */{ 7,  7,  7,  0,  0,  7,  7,  7,  0,  7,  7,  7,  0,  7,  7,  7,  6,  0},
                             /* M */{ 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 42,  0, 43,  0,  0},
                             /* K */{14, 14, 14,  0,  0, 14, 14, 14, 14, 14, 14, 14,  0, 14, 14, 14,  0,  0},
                             /* k */{16, 16, 16,  0, 15, 16, 16, 16, 16, 16, 16, 16,  0, 16, 16, 16,  0,  0},
                             /* N */{19, 20, 21,  0,  0, 22, 23, 24, 25, 26, 27, 28,  0, 18, 17, 18,  0,  0},
                             /* n */{29, 29, 29,  0, 29, 29, 29, 29, 29, 29, 29, 29, 30, 29, 29, 29,  0,  0},
                             /* F */{44, 44, 44, 44,  0, 46, 47, 45, 44, 48,  0, 44, 44, 44, 44, 44, 44, 44},
                             /* P */{ 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 49,  0,  0,  0,  0},
                             /* p */{ 0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 50, 51,  0, 57,  0,  0,  0,  0},
                             /* U */{ 0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 52,  0,  0, 53,  0,  0,  0,  0},
                             /* u */{32, 33, 34,  0,  0, 35, 36, 37, 38, 39, 40, 41,  0, 31, 55, 31,  0,  0}};

    private String[] formulas = {     "", "S>Es", "s>@$",  "s>|@|S", "E>LFE", "L>[@[l", "l>^K]@]", "l>K]@]", "L>(@(S)@)",  "L>M",
                                 "L>.@.",  "L>-",  "L>^", "L>\\@\\",  "K>Nk",     "k>",     "k>K",  "N>\\n",      "N>Mn", "N>(n",
                                  "N>)n", "N>.n", "N>+n",    "N>*n",  "N>?n",   "N>|n",    "N>{n",   "N>}n",      "N>,n",   "n>",
                                  "n>-u",  "u>M",  "u>(",     "u>)",   "u>.",    "u>+",     "u>*",    "u>?",       "u>|",  "u>{",
                                   "u>}",  "u>,",  "M>D",     "M>C",    "F>",  "F>?@?",   "F>+@+",  "F>*@*", "F>{@{P}@}", "P>Dp",
                                    "p>", "p>,U",   "U>",     "U>D",   "L>,",   "u>\\",      "E>",   "p>Dp"};

    public AutomatSintaxRuleAnalyser() {
        super("dsuyasov.inetDoc.automat.AutomatSintaxRuleAnalyser");
    }

    public void startAutomat(String regularExpression, boolean byAho) {
        lexicalAnalyser = new AutomatLexicalRuleAnalyser();
        this.regularExpression = regularExpression;
        this.byAho = byAho;
        work();
    }

    public void initialize() {
        z0_initializeAll();
        lexicalAnalyser.startAutomat(regularExpression);
    }

    public void automat(int event) {
        int y_statePrevious = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_STACK_ANALYSIS:
                if (x0_lexIsError()) {
                    y_state = STATE_3_ERROR;
                    log.automatChangeState(automatName, 0, 3);
                } else if (x1_ommandOnStackTop()) {
                    z1_realizeTheCommand();
                } else if (x2_terminalOnStackTop()) {
                    y_state = STATE_1_TERMINAL;
                    log.automatChangeState(automatName, 0, 1);
                } else if (x3_nonterminalOnStackTop()) {
                    y_state = STATE_2_NONTERMINAL;
                    z3_findFormula();
                    log.automatChangeState(automatName, 0, 2);
                } else {
                    y_state = STATE_3_ERROR;
                    log.automatChangeState(automatName, 0, 3);
                }
                break;
            case STATE_1_TERMINAL:
                if (x4_stackAndLexHasEndSymbol()) {
                    z5_finishTheWork();
                } else if (x5_stackTopAndLexEqual()) {
                    z2_takeTerminalFromStack();
                    lexicalAnalyser.work();
                    y_state = STATE_0_STACK_ANALYSIS;
                    log.automatChangeState(automatName, 1, 0);
                } else {
                    y_state = STATE_3_ERROR;
                    log.automatChangeState(automatName, 1, 3);
                }
                break;
            case STATE_2_NONTERMINAL:
                if (x6_formulaExists()) {
                    z4_setFormulaToStack();
                    y_state = STATE_0_STACK_ANALYSIS;
                    log.automatChangeState(automatName, 2, 0);
                } else {
                    y_state = STATE_3_ERROR;
                    log.automatChangeState(automatName, 2, 3);
                }
                break;
            case STATE_3_ERROR:
                z5_finishTheWork();
                break;
        }

        if (y_state != y_statePrevious)
            switch (y_state) {
                case STATE_0_STACK_ANALYSIS:

                    break;
                case STATE_1_TERMINAL:

                    break;
                case STATE_2_NONTERMINAL:

                    break;
                case STATE_3_ERROR:
                    z6_saveErrorMessage();
                    z5_finishTheWork();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }

    //  

    private boolean x0_lexIsError() {
        boolean result = lexicalAnalyser.lexType.charAt(0) == 'E';
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_ommandOnStackTop() {
        boolean result = ((Character)stack.peek()).charValue() == '@';
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_terminalOnStackTop() {
        boolean result = terminals.indexOf(((Character)stack.peek()).charValue()) >= 0;
        log.inputVariable(automatName, 2, result);
        return result;
    }

    private boolean x3_nonterminalOnStackTop() {
        boolean result = nonterminals.indexOf(((Character)stack.peek()).charValue()) >= 0;
        log.inputVariable(automatName, 3, result);
        return result;
    }

    private boolean x4_stackAndLexHasEndSymbol() {
        char endSymbol = terminals.charAt(terminals.length() - 1);
        boolean result = (endSymbol == ((Character) stack.peek()).charValue()) &&
                         (endSymbol == lexicalAnalyser.lexType.charAt(0));
        log.inputVariable(automatName, 4, result);
        return result;
    }

    private boolean x5_stackTopAndLexEqual() {
        boolean result = (((Character) stack.peek()).charValue() == lexicalAnalyser.lexType.charAt(0));
        log.inputVariable(automatName, 5, result);
        return result;
    }

    private boolean x6_formulaExists() {
        boolean result = formula.length() != 0;
        log.inputVariable(automatName, 6, result);
        return result;
    }

    //  

    private void z0_initializeAll() {
        log.output(automatName, 0);
        stack.clear();
        stack.push(new Character(terminals.charAt(terminals.length() - 1)));
        stack.push(new Character(nonterminals.charAt(0)));
        formula = "";
        expression = "";
        if (byAho)
            ruleBuilder = new TreeBuilder();
        else
            ruleBuilder = new RuleBuilder();
    }

    private void z1_realizeTheCommand() {
        log.output(automatName, 1);
        stack.pop();
        try {
            ruleBuilder.command(((Character) stack.pop()).charValue(), expression);
        } catch (Exception re) {
            ERROR = re.getMessage();
        } finally {
           expression = "";
        }
    }

    private void z2_takeTerminalFromStack() {
        log.output(automatName, 2);
        stack.pop();
        expression += ((lexicalAnalyser.lexType.equals("\\"))?"\\":"") + lexicalAnalyser.lexValue;
    }

    private void z3_findFormula() {
        log.output(automatName, 3);
        int terminalNumber = terminals.indexOf(lexicalAnalyser.lexType.charAt(0));
        int nonTerminalNumber = nonterminals.indexOf(((Character)stack.pop()).charValue());
        int formulaNumber = table[nonTerminalNumber][terminalNumber];
        formula = formulas[formulaNumber];
    }

    private void z4_setFormulaToStack() {
        log.output(automatName, 4);
        if (formula.length() > 2) {
            formula = formula.substring(2);
            for (int iLoop = formula.length() - 1; iLoop >= 0; iLoop--)
                stack.push(new Character(formula.charAt(iLoop)));
        }
    }

    private void z5_finishTheWork() {
        log.output(automatName, 5);
        running = false;
    }

    private void z6_saveErrorMessage() {
        log.output(automatName, 6);
        if (lexicalAnalyser.lexType.charAt(0) == 'E')
            ERROR = lexicalAnalyser.lexValue;
        else
            ERROR = ruleBuilder.getError();
        if (ERROR.length() == 0)
            ERROR = "       ["+regularExpression+"]     ["+lexicalAnalyser.lexValue+"]";
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return "    S$";
            case 1: return "   ";
            case 2: return "   ";
            case 3: return "  ";
            case 4: return "    ";
            case 5: return "  ";
            case 6: return "   ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "  - ";
            case 1: return "   ";
            case 2: return "   ";
            case 3: return "   ";
            case 4: return "        ";
            case 5: return "        ";
            case 6: return "     ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "";
            case 2: return "";
            case 3: return "";
        }
        return "    ["+number.intValue()+"]";
    }

}
