/**
 @author Dmitriy I. Suyasov
 @since 31.08.2005 12:32:15
 */
package dsuyasov.inetDoc.functionality;

import dsuyasov.inetDoc.automat.Automat;

public class RuleProcessor extends Automat {

    public static final int AUTOMAT_NUMBER = 4;

    public static final int STATE_0_PROCESS_CHILDREN_RULES_START = 0;
    public static final int STATE_1_PROCESS_PARENT_RULE = 1;
    public static final int STATE_2_CLOSE_AUTOMAT = 2;

    public static final int NOBODY_SUCCESSED = 0;
    public static final int PARENT_SUCCESSED = 1;
    public static final int CHILD_SUCCESSED = 2;

    public int successedResult = 0;
    public int successedParameter = 0;

    private RulesStarter rulesStarter;
    private String str;
    private Object parentResult = null;

    public RuleProcessor() {
        super("dsuyasov.inetDoc.functionality.RuleProcessor");
    }

    public void startAutomat(RulesStarter rulesStarter) {
        this.rulesStarter = rulesStarter;
        this.str = rulesStarter.getStringStack();
        work();
    }

    public void startAutomat(RulesStarter rulesStarter, String inputString) {
        this.rulesStarter = rulesStarter;
        this.str = inputString;
        work();
    }

    /*@
          
    */
    protected void initialize() {
        z0_initialiseResults();
        rulesStarter.startAutomat(str);
    }

    /*@
          
        @param event   
    */
    protected void automat(int event) {
        int y_previousState = y_state;
        log.addAutomatStartWork(automatName, y_state);
        switch (y_state) {
            case STATE_0_PROCESS_CHILDREN_RULES_START:
                y_state = STATE_1_PROCESS_PARENT_RULE;
                log.automatChangeState(automatName, 0, 1);
                break;
            case STATE_1_PROCESS_PARENT_RULE:
                if (x0_parentIsSuccessed()) {
                    if (x1_childIsSuccessed())
                        if (x2_parentHasHighPriority()) {
                            z2_setParentVictory();
                        } else {
                            z3_setChildVictory();
                        }
                    else {
                        z2_setParentVictory();
                    }
                } else if (x1_childIsSuccessed()) {
                    z3_setChildVictory();
                } else {
                    z4_setNobodyVictory();
                }
                y_state = STATE_2_CLOSE_AUTOMAT;
                log.automatChangeState(automatName, 1, 2);
                break;
            case STATE_2_CLOSE_AUTOMAT:
                z5_closeAutomat();
                break;
        }

        if (y_previousState != y_state)
            switch (y_state) {
                case STATE_1_PROCESS_PARENT_RULE:
                    z1_processParentRuleEnd();
                    break;
                case STATE_2_CLOSE_AUTOMAT:
                    z5_closeAutomat();
                    break;
            }
        log.addAutomatFinishWork(automatName, y_state);
    }


    //  

    private boolean x0_parentIsSuccessed() {
        boolean result = parentResult instanceof String;
        log.inputVariable(automatName, 0, result);
        return result;
    }

    private boolean x1_childIsSuccessed() {
        boolean result = rulesStarter.ruleNumber >= 0;
        log.inputVariable(automatName, 1, result);
        return result;
    }

    private boolean x2_parentHasHighPriority() {
        boolean result = true;
        int parentLength = str.length() - ((String) parentResult).length();
        int childLength = rulesStarter.resultString.length();
        if (parentLength == childLength) {
            Integer parentPriority = rulesStarter.parent.getEndPriority();
            Integer childPriority = ((RuleStarter) rulesStarter.rules.get(rulesStarter.ruleNumber)).getStartPriority();
            if (parentPriority == null) {
                if (childPriority != null) {
                    result = false;
                }
            } else {
                if (childPriority != null) {
                    result = parentPriority.intValue() < childPriority.intValue();
                }
            }
        } else if (parentLength < childLength) {
            result = false;
        }

        log.inputVariable(automatName, 2, result);
        return result;
    }

    //  

    private void z0_initialiseResults() {
        log.output(automatName, 0);
        successedResult = 0;
        successedParameter = 0;
    }

    private void z1_processParentRuleEnd() {
        log.output(automatName, 1);
        parentResult = (rulesStarter.parent == null)?Boolean.FALSE:rulesStarter.parent.processEnd(str);
    }

    private void z2_setParentVictory() {
        log.output(automatName, 2);
        successedResult = PARENT_SUCCESSED;
        successedParameter = str.length() - ((String) parentResult).length();

    }

    private void z3_setChildVictory() {
        log.output(automatName, 3);
        successedResult = CHILD_SUCCESSED;
        successedParameter = rulesStarter.ruleNumber;
    }

    private void z4_setNobodyVictory() {
        log.output(automatName, 4);
        successedResult = NOBODY_SUCCESSED;
    }

    private void z5_closeAutomat() {
        log.output(automatName, 5);
        running = false;
    }

    //   

    public static String outputs(Integer number) {
        switch (number.intValue()) {
            case 0: return " ";
            case 1: return "      ";
            case 2: return "    ";
            case 3: return "    ";
            case 4: return ",      ";
            case 5: return "  ";
        }
        return "   c  ["+number.intValue()+"]";
    }

    public static String inputVariables(Integer number) {
        switch (number.intValue()) {
            case 0: return "     ";
            case 1: return "     ";
            case 2: return "   ";
        }
        return "     ["+number.intValue()+"]";
    }

    public static String events(Integer number) {
        return "    ["+number.intValue()+"]";
    }

    public static String states(Integer number) {
        switch (number.intValue()) {
            case 0: return "  ";
            case 1: return "  ";
            case 2: return "  ";
        }
        return "    ["+number.intValue()+"]";
    }
}
